;**********************************************************************
; Integrated_Amp_S1.asm 
; Control Software for Integrated Amplifier S1 project 
; 
; Controls On/Off, Power supply, Source
; 
;
; Usage of Pins:
; GP5: relais for power supply
; GP4: switch Source select
; GP3: PWR-Switch (on/off)
; GP2: In 3 output
; GP1: In 2 output
; GP0: In 1 output
;
; Assuming INTOSC f=4MHz
;-------------------------------
;Stephan Asmus, Germany
;(c) stephan.asmus@t-online.de
;
; the software or parts of it may be used for private, non-commercial use
; the original author has to be named in such a case, however
;
; Das Programm oder Teile davon dürfen für private, nicht-kommerzielle
; Anwendungen weiterverwendet werden.
; Der ursprüngliche Autor ist dabei zu erwähnen
;
; 6/2022
;-------------------------------
; 29.06.2022	Starting
; 29.06.2022 	Initial Version, working as expected
; 21.07.2022	prolong wait-routine for better Key handling
; 
; 
;**********************************************************************
	LIST   P=PIC12F675

	#include <p12f675.inc>	; processor specific variable definitions


errorlevel  -302			;No bank select warnings

	__CONFIG (_CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _CPD_OFF & _MCLRE_OFF)


;***** VARIABLE DEFINITIONS
	cblock 0x20			;define a series of variables.
		d1				; Register to hold Delayloop	
		d2				; dto.
		d3
		source			;input selection
		eprom_adr		;eeprom Adress temp
		eprom_dat		;eeprom data temp
		
        endc

#define PWR_out	GPIO,5	;output to power supply relais
#define	SRC		GPIO,4	;Source selection Switch
#define	PWR		GPIO,3	;PWR switch


; **********************************************************************
; **********************************************************************
	ORG		0x000			; processor reset vector
	goto	init			; go to beginning of program

	ORG		0x004			; interrupt vector location
							;no interrupts

; ------------------------------------------------------------------------------
; START OF CODE to initialize the processor (Variables, I/O ...)
; ------------------------------------------------------------------------------
init
	bcf		STATUS,RP0		; Select Bank 0

	;initialise variables
	movlw 	07h
	movwf 	CMCON			;comparator off
	clrf	GPIO			;Init I/O-Port by clearing the outp. latches.

	bsf		STATUS, RP0		;Switch to BANK 1
	clrf 	ANSEL			;all digital IO
	movlw	b'00011000'		;configure in/output (GP5,GP2:GP0 output;
	movwf	TRISIO			;...GP4:GP3: input)
	movlw	b'00010000'
	movwf	WPU				;weak pullup at GP4
	bcf		OPTION_REG,7	;enable global weak pullup
	movlw	b'00001000'		;interrupt on change for GP3
	movwf	IOC

	
	;calibrating the oscillator...
	call	3FFh 			;Get the cal value
	movwf	OSCCAL 			;Calibrate it

	bcf		STATUS, RP0		;Back to BANK 0

	bcf		INTCON,GIE		;no global interrupts
	bsf		INTCON,GPIE		;Interrupt on Change enabled
	bcf		INTCON,GPIF		;clear interrupt flag
	
        
; ******************************************************************************
; MAIN, here we go...
; 
; ******************************************************************************
startup
	bsf		PWR_out			;power relais on
	btfss	PWR				;wait for key release
	goto	$-1
	call 	delay1s			;and wait to settle
	call 	delay1s			;and wait to settle
	
	;load preferences from eeprom
	
	movlw	0x00			;set adress for read of config
	movwf	eprom_adr		;save to adress
	call 	eprom_read		;load old config
	movwf	source			;save to current source
	;check plausibility of preset (has to be either 1, 2 or 4)
	;normally only needed at 1st startup cause data of eeprom is invalid after
	;programming of chip
	xorlw	0x01			;check if 1
	btfsc	STATUS,Z
	goto	startup1		;is 1, so leave
	xorlw	0x01
	xorlw	0x02			;check if 2
	btfsc	STATUS,Z		;
	goto	startup1		;is 2, so leave
	xorlw	0x02
	xorlw	0x04			;check if 4
	btfsc	STATUS,Z		;
	goto	startup1		;is 4, so leave
	;nothing of them, so set to default 1
	movlw	0x01
	movwf	source

startup1
	movf	source,w
	
	iorlw	b'00100000'		;set pwrout
	movwf	GPIO			;and output
		
mainloop
	btfsc	SRC				;source pressed?
	goto	main1			;no than go on
	bcf		STATUS,C		;clear carry... just to be sure
	rlf		source,f		;next input
	btfsc	source,3		;input 4 selected? (which does not exist)
	bsf		source,0		;yes, then back to 1 (Bit 0)
	movlw	b'00000111'
	andwf	source,f		;and mask all than Bits 0..2
	movf	source,w		;get source
	iorlw	b'00100000'		;set pwrout
	movwf	GPIO			;and output
	call	delay100ms
	btfss	SRC				;wait for key release
	goto	$-1
	call	delay100ms		;wait for debounce
	call	delay100ms
	call	delay100ms
	

main1
	btfss	PWR				;pwr pressed?
	goto	do_sleep		;yes, then off & sleep
	
	goto 	mainloop		;start over


;-----------------------------------------------------------------
; Do Sleep
; Sets PIC to sleep after saving prefs
;-----------------------------------------------------------------
do_sleep
	;save preferences to eeprom
	clrf	eprom_adr			;Adress to save to 
	movf	source,w
	andlw	b'00000111'			;load current status and mask non needed bits
	movwf	eprom_dat
	call	eprom_write			;save to eprom
	
	bcf		GPIO,0				;disable inputs
	bcf		GPIO,1
	bcf		GPIO,2
	call	delay100ms
	bcf		PWR_out				;disable powerstage
	call	delay100ms
	call	delay100ms
	btfss	PWR					;wait for key release
	goto	$-1
	call	delay1s
	bcf		INTCON,GPIF			;clear INT Flag
	movf	GPIO,w				;read GPIO to disable mismatch condition
	btfsc	INTCON,GPIF			;really clear?
	goto	$-3
	
	sleep						;now goto sleep
								;we sleep until PWR is pressed (GP3)
	
	nop
	movf	GPIO,w				;read GPIO to disable mismatch condition
	bcf		INTCON,GPIF			;clear INT Flag
	goto	startup				;and go on


;---------------------------------------------------------------
;EEPROM handling routines
;---------------------------------------------------------------
;eprom read: Address in eprom_adr, return value in w
;---------------------------------------------------------------
eprom_read
	bcf		PIR1, EEIF		;clear interrupt flag (necessary?)
	movf	eprom_adr,w		;Set adress of read
	banksel	EEADR
	movwf	EEADR
	
	banksel	EECON1
	bsf		EECON1,RD		;init read
	banksel	EEDATA
	movf	EEDATA,w
	bcf		STATUS,RP0		;back to bank 0 (our standard)
	return
;-------------------------------------------------------------------------
;eprom write Address in eprom_adr, data in eprom_dat returns when finished
;-------------------------------------------------------------------------
eprom_write
	bcf		PIR1, EEIF		;clear interrupt (to be sure)
	movf	eprom_adr,w		;load Adress
	banksel	EEADR
	movwf	EEADR
	banksel	0x00
	movf	eprom_dat,w	;load value
	banksel	EEDATA
	movwf	EEDATA
	banksel	EECON1
	bsf		EECON1,WREN		;enable write
	
	bcf		INTCON,GIE		;we don't want interrupts now
	btfsc	INTCON,GIE		;make sure it is really cleared
	goto	$-2
	movlw	0x55			;see datasheet, required sequence
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf		EECON1,WR		;set write bit to begin write
	
	banksel	0x00			;back to Bank0 (our default)
	btfss	PIR1,EEIF		;wait for flag to be set (polling)
	goto	$-1
	bcf		PIR1,EEIF		;clear it, write complete

	banksel	EECON1
	bcf		EECON1,WREN		;clear write enable
	banksel	0x00

	return
; ------------------------------------------------------------------------------
; delay: several delay routines used in the program
; ------------------------------------------------------------------------------
delay1s
			;999990 cycles
	movlw	0x07
	movwf	d1
	movlw	0x2F
	movwf	d2
	movlw	0x03
	movwf	d3
delay1s_0
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	$+2
	decfsz	d3, f
	goto	delay1s_0
			;6 cycles
	goto	$+1
	goto	$+1
	goto	$+1
			;4 cycles (including call)
	return
;----
delay100ms
			;99993 cycles
	movlw	0x1E
	movwf	d1
	movlw	0x4F
	movwf	d2
delay_1
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	delay_1
			;3 cycles
	goto	$+1
	nop
			;4 cycles (including call)
	return
;----
delay10us
			;6 cycles
	goto	$+1
	goto	$+1
	goto	$+1

			;4 cycles (including call)
	return
; *****************************************************************
	END					;'end of program''
